@@ -17,7 +17,7 @@ use std::{
 use tap::prelude::*;
 
 /// 七牛配置信息
-#[derive(Serialize, Deserialize, PartialEq, Eq, Debug)]
+#[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Debug, Default)]
 pub struct Config {
     #[serde(alias = "ak")]
     access_key: String,
@@ -45,16 +45,8 @@ pub struct Config {
     base_timeout_ms: Option<u64>,
     dial_timeout_ms: Option<u64>,
 }
-static QINIU_CONFIG: Lazy<RwLock<Option<Config>>> = Lazy::new(|| {
-    RwLock::new(load_config()).tap(|_| {
-        on_config_updated(|| {
-            if let Some(config) = load_config() {
-                *QINIU_CONFIG.write().unwrap() = Some(config);
-            }
-            info!("QINIU_CONFIG reloaded: {:?}", QINIU_CONFIG);
-        })
-    })
-});
+static QINIU_CONFIG: Lazy<RwLock<Option<Config>>> = Lazy::new(|| RwLock::new(load_config()));
+
 pub(super) static HTTP_CLIENT: Lazy<RwLock<HTTPClient>> = Lazy::new(|| {
     RwLock::new(build_http_client()).tap(|_| {
         on_config_updated(|| {
@@ -72,6 +64,12 @@ pub fn is_qiniu_enabled() -> bool {
     QINIU_CONFIG.read().unwrap().is_some()
 }
 
+/// 手动设置七牛环境配置
+#[inline]
+pub fn set_qiniu_config(config: Config) {
+    set_config_and_reload(config)
+}
+
 fn build_http_client() -> HTTPClient {
     let mut base_timeout_ms = 3000u64;
     let mut dial_timeout_ms = 50u64;
@@ -99,6 +97,7 @@ fn build_http_client() -> HTTPClient {
 }
 
 const QINIU_ENV: &str = "QINIU";
+const QINIU_DISABLE_CONFIG_HOT_RELOADING_ENV: &str = "QINIU_DISABLE_CONFIG_HOT_RELOADING";
 
 fn load_config() -> Option<Config> {
     if let Ok(qiniu_config_path) = env::var(QINIU_ENV) {
@@ -109,92 +108,109 @@ fn load_config() -> Option<Config> {
                 serde_json::from_slice(&qiniu_config).ok()
             };
             if let Some(qiniu_config) = qiniu_config {
-                setup_config_watcher(&qiniu_config_path).ok();
-                return Some(qiniu_config);
+                if env::var_os(QINIU_DISABLE_CONFIG_HOT_RELOADING_ENV).is_none() {
+                    setup_config_watcher(&qiniu_config_path).ok();
+                }
+                Some(qiniu_config)
             } else {
                 error!(
                     "Qiniu config file cannot be deserialized: {}",
                     qiniu_config_path
                 );
-                return None;
+                None
             }
         } else {
             error!("Qiniu config file cannot be open: {}", qiniu_config_path);
-            return None;
+            None
         }
     } else {
         warn!("QINIU Env IS NOT ENABLED");
-        return None;
+        None
+    }
+}
+
+fn setup_config_watcher(config_path: impl Into<PathBuf>) -> IOResult<()> {
+    let config_path = config_path
+        .into()
+        .canonicalize()
+        .tap_err(|err| warn!("Failed to canonicalize config path: {:?}", err))?;
+
+    static UNIQUE_THREAD: OnceCell<JoinHandle<()>> = OnceCell::new();
+
+    if let Err(err) = UNIQUE_THREAD.get_or_try_init(|| {
+        ThreadBuilder::new()
+            .name("qiniu-config-watcher".into())
+            .spawn(move || {
+                if let Err(err) = setup_config_watcher_inner(&config_path) {
+                    error!("Qiniu config file watcher was setup failed: {:?}", err);
+                }
+            })
+    }) {
+        error!(
+            "Failed to start thread to watch Qiniu config file: {:?}",
+            err
+        );
     }
 
-    fn setup_config_watcher(config_path: impl Into<PathBuf>) -> IOResult<()> {
-        let config_path = config_path
-            .into()
-            .canonicalize()
-            .tap_err(|err| warn!("Failed to canonicalize config path: {:?}", err))?;
+    return Ok(());
 
-        static UNIQUE_THREAD: OnceCell<JoinHandle<()>> = OnceCell::new();
+    fn setup_config_watcher_inner(config_path: &Path) -> NotifyResult<()> {
+        let (tx, rx) = channel();
+        let mut watcher = watcher(tx, Duration::from_millis(500))?;
+        watcher.watch(
+            config_path.parent().unwrap_or_else(|| Path::new("/")),
+            RecursiveMode::NonRecursive,
+        )?;
 
-        if let Err(err) = UNIQUE_THREAD.get_or_try_init(|| {
-            ThreadBuilder::new()
-                .name("qiniu-config-watcher".into())
-                .spawn(move || {
-                    if let Err(err) = setup_config_watcher_inner(&config_path) {
-                        error!("Qiniu config file watcher was setup failed: {:?}", err);
+        info!("Qiniu config file watcher was setup");
+
+        loop {
+            match rx.recv() {
+                Ok(event) => match event {
+                    DebouncedEvent::Create(ref path) if path == config_path => {
+                        event_received(event);
                     }
-                })
-        }) {
-            error!(
-                "Failed to start thread to watch Qiniu config file: {:?}",
-                err
-            );
-        }
-
-        return Ok(());
-
-        fn setup_config_watcher_inner(config_path: &Path) -> NotifyResult<()> {
-            let (tx, rx) = channel();
-            let mut watcher = watcher(tx, Duration::from_millis(500))?;
-            watcher.watch(
-                config_path.parent().unwrap_or_else(|| Path::new("/")),
-                RecursiveMode::NonRecursive,
-            )?;
-
-            info!("Qiniu config file watcher was setup");
-
-            loop {
-                match rx.recv() {
-                    Ok(event) => match event {
-                        DebouncedEvent::Create(ref path) if path == config_path => {
-                            event_received(event);
-                        }
-                        DebouncedEvent::Write(ref path) if path == config_path => {
-                            event_received(event);
-                        }
-                        DebouncedEvent::Error(err, _) => {
-                            error!(
-                                "Received error event from Qiniu config file watcher: {:?}",
-                                err
-                            );
-                        }
-                        _ => {}
-                    },
-                    Err(err) => {
+                    DebouncedEvent::Write(ref path) if path == config_path => {
+                        event_received(event);
+                    }
+                    DebouncedEvent::Error(err, _) => {
                         error!(
-                            "Failed to receive event from Qiniu config file watcher: {:?}",
+                            "Received error event from Qiniu config file watcher: {:?}",
                             err
                         );
                     }
+                    _ => {}
+                },
+                Err(err) => {
+                    error!(
+                        "Failed to receive event from Qiniu config file watcher: {:?}",
+                        err
+                    );
                 }
             }
         }
+    }
 
-        fn event_received(event: DebouncedEvent) {
-            info!("Received event {:?} from Qiniu config file watcher", event);
-            for handle in CONFIG_UPDATE_HANDLERS.read().unwrap().iter() {
-                handle();
-            }
-        }
+    #[inline]
+    fn event_received(event: DebouncedEvent) {
+        info!("Received event {:?} from Qiniu config file watcher", event);
+        reload_config();
+    }
+}
+
+#[inline]
+fn reload_config() {
+    if let Some(config) = load_config() {
+        set_config_and_reload(config);
+    }
+}
+
+#[inline]
+fn set_config_and_reload(config: Config) {
+    *QINIU_CONFIG.write().unwrap() = Some(config);
+    info!("QINIU_CONFIG reloaded: {:?}", QINIU_CONFIG);
+    for handle in CONFIG_UPDATE_HANDLERS.read().unwrap().iter() {
+        handle();
     }
 }
 
@@ -318,17 +334,7 @@ impl ConfigBuilder {
                 secret_key: secret_key.into(),
                 bucket: bucket.into(),
                 io_urls,
-                uc_urls: None,
-                monitor_urls: None,
-                sim: None,
-                normalize_key: None,
-                private: None,
-                retry: None,
-                punish_time_s: None,
-                base_timeout_ms: None,
-                dial_timeout_ms: None,
-                dot_interval_s: None,
-                max_dot_buffer_size: None,
+                ..Default::default()
             },
         }
     }
@@ -339,14 +345,42 @@ impl ConfigBuilder {
         self.inner
     }
 
-    /// 配置 UC 服务器域名列表
+    /// 配置七牛 Access Key
+    #[inline]
+    pub fn access_key(mut self, access_key: impl Into<String>) -> Self {
+        self.inner.access_key = access_key.into();
+        self
+    }
+
+    /// 配置七牛 Secret Key
+    #[inline]
+    pub fn secret_key(mut self, secret_key: impl Into<String>) -> Self {
+        self.inner.secret_key = secret_key.into();
+        self
+    }
+
+    /// 配置七牛存储空间
+    #[inline]
+    pub fn bucket(mut self, bucket: impl Into<String>) -> Self {
+        self.inner.bucket = bucket.into();
+        self
+    }
+
+    /// 配置 IO 服务器 URL 列表
+    #[inline]
+    pub fn io_urls(mut self, io_urls: Option<Vec<String>>) -> Self {
+        self.inner.io_urls = io_urls;
+        self
+    }
+
+    /// 配置 UC 服务器 URL 列表
     #[inline]
     pub fn uc_urls(mut self, uc_urls: Option<Vec<String>>) -> Self {
         self.inner.uc_urls = uc_urls;
         self
     }
 
-    /// 配置监控服务器域名列表，如果不配置或配置为空，则不会启用打点功能
+    /// 配置监控服务器 URL 列表，如果不配置或配置为空，则不会启用打点功能
     #[inline]
     pub fn monitor_urls(mut self, monitor_urls: Option<Vec<String>>) -> Self {
         self.inner.monitor_urls = monitor_urls;
@@ -419,11 +453,216 @@ impl ConfigBuilder {
     }
 }
 
+impl Config {
+    /// 获取七牛 Access Key
+    #[inline]
+    pub fn access_key(&self) -> &str {
+        &self.access_key
+    }
+
+    /// 设置七牛 Access Key
+    #[inline]
+    pub fn set_access_key(&mut self, access_key: impl Into<String>) -> &mut Self {
+        self.access_key = access_key.into();
+        self
+    }
+
+    /// 获取七牛 Secret Key
+    #[inline]
+    pub fn secret_key(&self) -> &str {
+        &self.secret_key
+    }
+
+    /// 设置七牛 Secret Key
+    #[inline]
+    pub fn set_secret_key(&mut self, secret_key: impl Into<String>) -> &mut Self {
+        self.secret_key = secret_key.into();
+        self
+    }
+
+    /// 获取七牛存储空间
+    #[inline]
+    pub fn bucket(&self) -> &str {
+        &self.bucket
+    }
+
+    /// 设置七牛存储空间
+    #[inline]
+    pub fn set_bucket(&mut self, bucket: impl Into<String>) -> &mut Self {
+        self.bucket = bucket.into();
+        self
+    }
+
+    /// 获取 IO 服务器 URL 列表
+    #[inline]
+    pub fn io_urls(&self) -> Option<&[String]> {
+        self.io_urls.as_ref().map(|urls| urls.as_ref())
+    }
+
+    /// 设置 IO 服务器 URL 列表
+    #[inline]
+    pub fn set_io_urls(&mut self, io_urls: Option<impl Into<Vec<String>>>) -> &mut Self {
+        self.io_urls = io_urls.map(|urls| urls.into());
+        self
+    }
+
+    /// 获取 UC 服务器 URL 列表
+    #[inline]
+    pub fn uc_urls(&self) -> Option<&[String]> {
+        self.uc_urls.as_ref().map(|urls| urls.as_ref())
+    }
+
+    /// 设置 UC 服务器 URL 列表
+    #[inline]
+    pub fn set_uc_urls(&mut self, uc_urls: Option<impl Into<Vec<String>>>) -> &mut Self {
+        self.uc_urls = uc_urls.map(|urls| urls.into());
+        self
+    }
+
+    /// 获取监控服务器服务器 URL 列表
+    #[inline]
+    pub fn monitor_urls(&self) -> Option<&[String]> {
+        self.monitor_urls.as_ref().map(|urls| urls.as_ref())
+    }
+
+    /// 设置监控服务器 URL 列表
+    #[inline]
+    pub fn set_monitor_urls(&mut self, monitor_urls: Option<impl Into<Vec<String>>>) -> &mut Self {
+        self.monitor_urls = monitor_urls.map(|urls| urls.into());
+        self
+    }
+
+    /// 是否使用 Getfile API
+    #[inline]
+    pub fn use_getfile_api(&self) -> Option<bool> {
+        self.sim.map(|b| !b)
+    }
+
+    /// 设置是否使用 Getfile API
+    #[inline]
+    pub fn set_use_getfile_api(&mut self, use_getfile_api: Option<bool>) -> &mut Self {
+        self.sim = use_getfile_api.map(|b| !b);
+        self
+    }
+
+    /// 是否对 key 进行格式化
+    #[inline]
+    pub fn normalize_key(&self) -> Option<bool> {
+        self.normalize_key
+    }
+
+    /// 设置是否对 key 进行格式化
+    #[inline]
+    pub fn set_normalize_key(&mut self, normalize_key: Option<bool>) -> &mut Self {
+        self.normalize_key = normalize_key;
+        self
+    }
+
+    /// 是否使用私有存储空间
+    #[inline]
+    pub fn private(&self) -> Option<bool> {
+        self.private
+    }
+
+    /// 设置是否使用私有存储空间
+    #[inline]
+    pub fn set_private(&mut self, private: Option<bool>) -> &mut Self {
+        self.private = private;
+        self
+    }
+
+    /// 获取 IO 和 UC 服务器访问重试次数
+    #[inline]
+    pub fn retry(&self) -> Option<usize> {
+        self.retry
+    }
+
+    /// 设置 IO 和 UC 服务器访问重试次数
+    #[inline]
+    pub fn set_retry(&mut self, retry: Option<usize>) -> &mut Self {
+        self.retry = retry;
+        self
+    }
+
+    /// 获取打点记录上传频率
+    #[inline]
+    pub fn dot_interval(&self) -> Option<Duration> {
+        self.dot_interval_s.map(Duration::from_secs)
+    }
+
+    /// 设置打点记录上传频率
+    #[inline]
+    pub fn set_dot_interval(&mut self, dot_interval: Option<Duration>) -> &mut Self {
+        self.dot_interval_s = dot_interval.map(|d| d.as_secs());
+        self
+    }
+
+    /// 获取打点记录本地缓存文件尺寸上限
+    #[inline]
+    pub fn max_dot_buffer_size(&self) -> Option<u64> {
+        self.max_dot_buffer_size
+    }
+
+    /// 设置打点记录本地缓存文件尺寸上限
+    #[inline]
+    pub fn set_max_dot_buffer_size(&mut self, max_dot_buffer_size: Option<u64>) -> &mut Self {
+        self.max_dot_buffer_size = max_dot_buffer_size;
+        self
+    }
+
+    /// 获取域名访问失败后的惩罚时长
+    #[inline]
+    pub fn punish_time(&self) -> Option<Duration> {
+        self.punish_time_s.map(Duration::from_secs)
+    }
+
+    /// 设置域名访问失败后的惩罚时长
+    #[inline]
+    pub fn set_punish_time(&mut self, punish_time: Option<Duration>) -> &mut Self {
+        self.punish_time_s = punish_time.map(|d| d.as_secs());
+        self
+    }
+
+    /// 获取域名访问的基础超时时长
+    #[inline]
+    pub fn base_timeout(&self) -> Option<Duration> {
+        self.base_timeout_ms.map(Duration::from_millis)
+    }
+
+    /// 设置域名访问失败后的惩罚时长
+    #[inline]
+    pub fn set_base_timeout(&mut self, base_timeout: Option<Duration>) -> &mut Self {
+        self.base_timeout_ms = base_timeout.map(|d| d.as_millis().try_into().unwrap_or(u64::MAX));
+        self
+    }
+
+    /// 获取域名连接的超时时长
+    #[inline]
+    pub fn connect_timeout(&self) -> Option<Duration> {
+        self.dial_timeout_ms.map(Duration::from_millis)
+    }
+
+    /// 设置域名连接的超时时长
+    #[inline]
+    pub fn set_connect_timeout(&mut self, connect_timeout: Option<Duration>) -> &mut Self {
+        self.dial_timeout_ms =
+            connect_timeout.map(|d| d.as_millis().try_into().unwrap_or(u64::MAX));
+        self
+    }
+}
+
+impl From<Config> for ConfigBuilder {
+    #[inline]
+    fn from(config: Config) -> Self {
+        ConfigBuilder { inner: config }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
+    use anyhow::Result;
     use std::{
-        error::Error,
         fs::{remove_file, rename, OpenOptions},
         io::Write,
         sync::atomic::{AtomicUsize, Ordering::Relaxed},
@@ -432,7 +671,7 @@ mod tests {
     use tempfile::Builder as TempFileBuilder;
 
     #[test]
-    fn test_load_config() -> Result<(), Box<dyn Error>> {
+    fn test_load_config() -> Result<()> {
         env_logger::try_init().ok();
 
         let mut config = Config {
@@ -440,17 +679,7 @@ mod tests {
             secret_key: "test-sk-1".into(),
             bucket: "test-bucket-1".into(),
             io_urls: Some(vec!["http://io1.com".into(), "http://io2.com".into()]),
-            uc_urls: Default::default(),
-            monitor_urls: Default::default(),
-            sim: Default::default(),
-            normalize_key: Default::default(),
-            private: Default::default(),
-            retry: Default::default(),
-            dot_interval_s: Default::default(),
-            max_dot_buffer_size: Default::default(),
-            punish_time_s: Default::default(),
-            base_timeout_ms: Default::default(),
-            dial_timeout_ms: Default::default(),
+            ..Default::default()
         };
         let tempfile_path = {
             let mut tempfile = TempFileBuilder::new().suffix(".toml").tempfile()?;
@@ -552,4 +781,115 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn test_set_config() {
+        env_logger::try_init().ok();
+
+        let mut config = Config {
+            access_key: "test-ak-1".into(),
+            secret_key: "test-sk-1".into(),
+            bucket: "test-bucket-1".into(),
+            io_urls: Some(vec!["http://io1.com".into(), "http://io2.com".into()]),
+            ..Default::default()
+        };
+
+        static UPDATED: AtomicUsize = AtomicUsize::new(0);
+        UPDATED.store(0, Relaxed);
+
+        on_config_updated(|| {
+            UPDATED.fetch_add(1, Relaxed);
+        });
+        on_config_updated(|| {
+            UPDATED.fetch_add(1, Relaxed);
+        });
+        on_config_updated(|| {
+            UPDATED.fetch_add(1, Relaxed);
+        });
+
+        set_qiniu_config(config.to_owned());
+        assert_eq!(UPDATED.load(Relaxed), 3);
+
+        config.access_key = "test-ak-2".into();
+        config.secret_key = "test-sk-2".into();
+        config.bucket = "test-bucket-2".into();
+
+        set_qiniu_config(config);
+        assert_eq!(UPDATED.load(Relaxed), 6);
+    }
+
+    #[test]
+    fn test_load_config_without_hot_reloading() -> Result<()> {
+        env_logger::try_init().ok();
+
+        struct QiniuHotReloadingEnvGuard;
+
+        impl QiniuHotReloadingEnvGuard {
+            #[inline]
+            fn new() -> Self {
+                env::set_var(QINIU_DISABLE_CONFIG_HOT_RELOADING_ENV, "1");
+                Self
+            }
+        }
+
+        impl Drop for QiniuHotReloadingEnvGuard {
+            #[inline]
+            fn drop(&mut self) {
+                env::remove_var(QINIU_DISABLE_CONFIG_HOT_RELOADING_ENV)
+            }
+        }
+
+        let _guard = QiniuHotReloadingEnvGuard::new();
+
+        let mut config = Config {
+            access_key: "test-ak-1".into(),
+            secret_key: "test-sk-1".into(),
+            bucket: "test-bucket-1".into(),
+            io_urls: Some(vec!["http://io1.com".into(), "http://io2.com".into()]),
+            ..Default::default()
+        };
+        let tempfile_path = {
+            let mut tempfile = TempFileBuilder::new().suffix(".toml").tempfile()?;
+            tempfile.write_all(&toml::to_vec(&config)?)?;
+            tempfile.flush()?;
+            env::set_var(QINIU_ENV, tempfile.path().as_os_str());
+            tempfile.into_temp_path()
+        };
+
+        static UPDATED: AtomicUsize = AtomicUsize::new(0);
+        UPDATED.store(0, Relaxed);
+
+        let loaded = load_config().unwrap();
+        assert_eq!(loaded, config);
+
+        on_config_updated(|| {
+            UPDATED.fetch_add(1, Relaxed);
+        });
+        on_config_updated(|| {
+            UPDATED.fetch_add(1, Relaxed);
+        });
+        on_config_updated(|| {
+            UPDATED.fetch_add(1, Relaxed);
+        });
+
+        sleep(Duration::from_secs(1));
+
+        config.access_key = "test-ak-2".into();
+        config.secret_key = "test-sk-2".into();
+        config.bucket = "test-bucket-2".into();
+
+        {
+            let mut tempfile = OpenOptions::new()
+                .write(true)
+                .truncate(true)
+                .open(&tempfile_path)?;
+            tempfile.write_all(&toml::to_vec(&config)?)?;
+            tempfile.flush()?;
+        }
+
+        sleep(Duration::from_secs(1));
+        assert_eq!(UPDATED.load(Relaxed), 0);
+
+        Ok(())
+    }
 }
