@@ -91,6 +91,7 @@ struct RangeReaderInner {
     tries: usize,
     use_getfile_api: bool,
     normalize_key: bool,
+    use_https: bool,
     private_url_lifetime: Option<Duration>,
 }
 
@@ -341,6 +342,7 @@ impl RangeReaderBuilder {
                 tries: self.io_tries,
                 use_getfile_api: self.use_getfile_api,
                 normalize_key: self.normalize_key,
+                use_https: self.use_https,
                 private_url_lifetime: self.private_url_lifetime,
             }),
             self.key,
@@ -419,6 +421,35 @@ impl RangeReader {
                 .map(|v| v.0)
         }
     }
+
+    /// 主动更新域名列表
+    ///
+    /// 如果返回为 true 表示更新成功，否则返回 false
+    pub fn update_urls(&self) -> bool {
+        self.inner.io_selector.update_hosts()
+    }
+
+    /// 获取当前可用的 IO 节点的域名
+    pub fn io_urls(&self) -> Vec<String> {
+        return self
+            .inner
+            .io_selector
+            .hosts()
+            .iter()
+            .map(|host| normalize_host(host, self.inner.use_https))
+            .collect();
+
+        #[inline]
+        fn normalize_host(host: &str, use_https: bool) -> String {
+            if host.contains("://") {
+                host.to_string()
+            } else if use_https {
+                "https://".to_owned() + host
+            } else {
+                "http://".to_owned() + host
+            }
+        }
+    }
 }
 
 impl ReadAt for RangeReader {
@@ -490,9 +521,12 @@ impl ReadAt for RangeReader {
     }
 }
 
+/// 通过 RangeReader::read_multi_ranges() 获取文件的区域以及对应的数据
 #[derive(Debug, Clone)]
 pub struct RangePart {
+    /// 区域对应的数据
     pub data: Vec<u8>,
+    /// 区域的开始偏移量和区域长度
     pub range: (u64, u64),
 }
 
@@ -547,7 +581,7 @@ impl RangeReader {
                                                     "Content-Type must be existed",
                                                 )
                                             })?;
-                                        extract_boundary(&content_type.to_str().map_err(|err| {
+                                        extract_boundary(content_type.to_str().map_err(|err| {
                                             IOError::new(IOErrorKind::InvalidInput, err)
                                         })?)
                                         .ok_or_else(|| {
@@ -1070,7 +1104,7 @@ impl RangeReader {
             if let Some(private_url_lifetime) = private_url_lifetime {
                 Url::parse(
                     &sign_download_url_with_lifetime(
-                        &credential,
+                        credential,
                         Url::parse(url).unwrap(),
                         private_url_lifetime,
                     )
@@ -1139,12 +1173,13 @@ mod tests {
     use super::{
         super::{
             cache_dir_path_of,
-            dot::{DotRecordKey, DotRecords, DotRecordsDashMap},
+            dot::{DotRecordKey, DotRecords, DotRecordsDashMap, DOT_FILE_NAME},
         },
         *,
     };
     use futures::{channel::oneshot::channel, future::join};
     use multipart::client::lazy::Multipart;
+    use serde_json::{json, to_vec as json_to_vec};
     use std::{
         boxed::Box,
         error::Error,
@@ -1180,13 +1215,13 @@ mod tests {
             tx.send(()).ok();
             handler.await.ok();
         }};
-        ($uc_addr:ident, $monitor_addr:ident, $uc_routes:ident, $records_map:ident, $code:block) => {{
-            let (uc_tx, uc_rx) = channel();
+        ($io_addr:ident, $monitor_addr:ident, $io_routes:ident, $records_map:ident, $code:block) => {{
+            let (io_tx, io_rx) = channel();
             let (monitor_tx, monitor_rx) = channel();
-            let ($uc_addr, uc_server) = warp::serve($uc_routes).bind_with_graceful_shutdown(
+            let ($io_addr, io_server) = warp::serve($io_routes).bind_with_graceful_shutdown(
                 ([127, 0, 0, 1], 0),
                 async move {
-                    uc_rx.await.ok();
+                    io_rx.await.ok();
                 },
             );
             let $records_map = Arc::new(DotRecordsDashMap::default());
@@ -1205,12 +1240,51 @@ mod tests {
                 .bind_with_graceful_shutdown(([127, 0, 0, 1], 0), async move {
                     monitor_rx.await.ok();
                 });
-            let uc_handler = spawn(uc_server);
+            let io_handler = spawn(io_server);
             let monitor_handler = spawn(monitor_server);
             $code;
-            uc_tx.send(()).ok();
+            io_tx.send(()).ok();
             monitor_tx.send(()).ok();
-            let _ = join(uc_handler, monitor_handler).await;
+            let _ = join(io_handler, monitor_handler).await;
+        }};
+        ($io_addr:ident, $uc_addr:ident, $io_routes:ident, $code:block) => {{
+            let (io_tx, io_rx) = channel();
+            let (uc_tx, uc_rx) = channel();
+            let ($io_addr, io_server) = warp::serve($io_routes).bind_with_graceful_shutdown(
+                ([127, 0, 0, 1], 0),
+                async move {
+                    io_rx.await.ok();
+                },
+            );
+            let io_addr = $io_addr.to_owned();
+            let uc_routes = {
+                path!("v4" / "query")
+                    .map(move || {
+                        Response::new(json_to_vec(&json!({
+                            "hosts": [{
+                                "ttl": 86400,
+                                "io": {
+                                    "domains": [io_addr]
+                                },
+                                "uc": {
+                                    "domains": []
+                                }
+                            }]
+                        })).unwrap().into())
+                    })
+            };
+            let ($uc_addr, uc_server) = warp::serve(uc_routes).bind_with_graceful_shutdown(
+                ([127, 0, 0, 1], 0),
+                async move {
+                    uc_rx.await.ok();
+                },
+            );
+            let io_handler = spawn(io_server);
+            let uc_handler = spawn(uc_server);
+            $code;
+            io_tx.send(()).ok();
+            uc_tx.send(()).ok();
+            let _ = join(io_handler, uc_handler).await;
         }};
     }
 
@@ -1242,8 +1316,8 @@ mod tests {
             action_1.or(action_2)
         };
 
-        starts_with_server!(uc_addr, monitor_addr, io_routes, records_map, {
-            let io_urls = vec![format!("http://{}", uc_addr)];
+        starts_with_server!(io_addr, monitor_addr, io_routes, records_map, {
+            let io_urls = vec![format!("http://{}", io_addr)];
             {
                 let downloader =
                     RangeReader::builder("bucket", "file", get_credential(), io_urls.to_owned())
@@ -1939,6 +2013,32 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn test_update_hosts() -> Result<(), Box<dyn Error>> {
+        env_logger::try_init().ok();
+        clear_cache()?;
+
+        let routes = { path!("file").map(move || Response::new("12345".into())) };
+        starts_with_server!(io_addr, uc_addr, routes, {
+            let io_urls = vec!["http://fakedomain:12345".to_owned()];
+            let uc_urls = vec![format!("http://{}", uc_addr)];
+            let downloader =
+                RangeReader::builder("bucket", "file", get_credential(), io_urls.to_owned())
+                    .uc_urls(uc_urls)
+                    .use_getfile_api(false)
+                    .normalize_key(true)
+                    .build();
+            spawn_blocking(move || {
+                assert_eq!(downloader.io_urls(), io_urls);
+                assert!(downloader.update_urls());
+                assert_eq!(downloader.io_urls(), vec![format!("http://{}", io_addr)]);
+                assert_eq!(&downloader.download().unwrap(), b"12345");
+            })
+            .await?;
+        });
+        Ok(())
+    }
+
     #[test]
     fn test_sign_download_url_with_deadline() -> Result<(), Box<dyn Error>> {
         env_logger::try_init().ok();
@@ -1964,7 +2064,7 @@ mod tests {
                 Err(err)
             }
         })?;
-        let dot_file_path = cache_dir_path_of("dot-file")?;
+        let dot_file_path = cache_dir_path_of(DOT_FILE_NAME)?;
         std::fs::remove_file(&dot_file_path).or_else(|err| {
             if err.kind() == IOErrorKind::NotFound {
                 Ok(())
