@@ -216,7 +216,7 @@ impl HostsUpdater {
 
         fn try_to_auto_update_in_thread(updater: Arc<HostsUpdater>) {
             if let Some(update_option) = &updater.update_option {
-                if let Ok(mut last_updated_at) = update_option.last_updated_at.try_lock() {
+                if let Ok(mut last_updated_at) = update_option.last_updated_at.lock() {
                     if last_updated_at.elapsed() >= update_option.interval {
                         if updater.update_hosts() {
                             info!("`host-selector-auto-updater` update hosts successfully");
@@ -428,6 +428,7 @@ impl HostSelectorBuilder {
     }
 }
 
+#[derive(Debug, Clone)]
 pub(super) struct HostInfo {
     pub(super) host: String,
     pub(super) timeout_power: usize,
@@ -440,6 +441,50 @@ impl HostSelector {
         HostSelectorBuilder::new(hosts)
     }
 
+    #[inline]
+    pub(super) fn set_hosts(&self, hosts: Vec<String>) {
+        self.hosts_updater.set_hosts(hosts)
+    }
+
+    #[inline]
+    pub(super) fn hosts(&self) -> Vec<String> {
+        self.hosts_updater
+            .hosts
+            .read()
+            .unwrap()
+            .iter()
+            .filter(|&host| {
+                self.hosts_updater
+                    .hosts_map
+                    .get(host)
+                    .map(|punished_info| {
+                        self.host_punisher.is_punishment_expired(&punished_info)
+                            || self.host_punisher.is_available(&punished_info)
+                    })
+                    .unwrap_or(true)
+            })
+            .cloned()
+            .collect()
+    }
+
+    #[inline]
+    pub(super) fn update_hosts(&self) -> bool {
+        if self.hosts_updater.update_hosts() {
+            info!("manual update hosts successfully");
+            if let Some(mut last_updated_at) = self
+                .hosts_updater
+                .update_option
+                .as_ref()
+                .and_then(|option| option.last_updated_at.lock().ok())
+            {
+                *last_updated_at = Instant::now();
+            }
+            true
+        } else {
+            false
+        }
+    }
+
     pub(super) fn select_host(&self) -> HostInfo {
         struct CurrentHostInfo<'a> {
             host: &'a str,
