@@ -5,7 +5,7 @@ use super::{
     HTTP_CLIENT,
 };
 use dashmap::DashMap;
-use log::{debug, warn};
+use log::{info, warn};
 use once_cell::sync::Lazy;
 use reqwest::StatusCode;
 use serde::{
@@ -16,8 +16,9 @@ use serde_json::{from_reader as json_from_reader, to_writer as json_to_writer};
 use std::{
     collections::HashMap,
     fmt,
-    fs::OpenOptions,
+    fs::{rename as rename_file, OpenOptions},
     io::{Error as IOError, ErrorKind as IOErrorKind, Result as IOResult},
+    path::Path,
     sync::Mutex,
     thread::spawn,
     time::{Duration, Instant, SystemTime},
@@ -89,6 +90,7 @@ struct ResponseBody {
 struct RegionResponseBody {
     ttl: u64,
     io: DomainsResponseBody,
+    uc: DomainsResponseBody,
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -119,6 +121,7 @@ impl HostsQuerier {
         }
     }
 
+    #[inline]
     pub(super) fn query_for_io_urls(
         &self,
         ak: &str,
@@ -138,6 +141,7 @@ impl HostsQuerier {
             .map(|domain| normalize_domain(domain, use_https))
             .collect());
 
+        #[inline]
         fn normalize_domain(domain: &str, use_https: bool) -> String {
             if domain.contains("://") {
                 domain.to_string()
@@ -196,9 +200,7 @@ impl HostsQuerier {
                 }
             });
         } else if modified {
-            spawn(move || {
-                let _ = save_cache();
-            });
+            spawn(save_cache);
         }
 
         Ok(cache_value.cached_response_body.to_owned())
@@ -217,7 +219,7 @@ fn query_for_domains_without_cache(
         uc_tries,
         dotter,
         |host, timeout_power, timeout| {
-            debug!(
+            info!(
                 "try to query hosts from {}, ak = {}, bucket = {}",
                 host,
                 ak.as_ref(),
@@ -257,6 +259,22 @@ fn query_for_domains_without_cache(
                             .map_err(|err| IOError::new(IOErrorKind::BrokenPipe, err))
                     }
                 })
+                .tap_ok(|body| {
+                    let uc_hosts: Vec<_> = body
+                        .hosts
+                        .first()
+                        .map(|host| {
+                            host.uc
+                                .domains
+                                .iter()
+                                .map(|domain| domain.to_string())
+                                .collect()
+                        })
+                        .expect("No host in uc query v4 response body");
+                    if !uc_hosts.is_empty() {
+                        uc_selector.set_hosts(uc_hosts);
+                    }
+                })
                 .map(|body| {
                     let min_ttl = body
                         .hosts
@@ -270,7 +288,7 @@ fn query_for_domains_without_cache(
                     }
                 })
                 .tap_ok(|_| {
-                    debug!(
+                    info!(
                         "update query cache for ak = {}, bucket = {} is successful",
                         ak.as_ref(),
                         bucket.as_ref(),
@@ -322,35 +340,76 @@ fn query_for_domains_without_cache(
     }
 }
 
+const CACHE_FILE_NAME: &str = "query-cache.json";
+const CACHE_TEMPFILE_NAME: &str = "query-cache.tmp.json";
+
 fn load_cache() -> IOResult<()> {
-    let cache_file_path = cache_dir_path_of("query-cache.json")?;
-    if let Ok(cache_file) = OpenOptions::new().read(true).open(&cache_file_path) {
-        let cache: HashMap<CacheKey, CacheValue> =
-            json_from_reader(cache_file).map_err(|err| IOError::new(IOErrorKind::Other, err))?;
-        CACHE_MAP.clear();
-        for (key, value) in cache.into_iter() {
-            CACHE_MAP.insert(key, value);
+    let cache_file_path = cache_dir_path_of(CACHE_FILE_NAME)?;
+    match OpenOptions::new().read(true).open(&cache_file_path) {
+        Ok(cache_file) => {
+            let cache: HashMap<CacheKey, CacheValue> = json_from_reader(cache_file)
+                .tap_err(|err| {
+                    warn!(
+                        "Failed to parse cache from cache file {:?}: {}",
+                        cache_file_path, err
+                    )
+                })
+                .map_err(|err| IOError::new(IOErrorKind::Other, err))?;
+            CACHE_MAP.clear();
+            for (key, value) in cache.into_iter() {
+                CACHE_MAP.insert(key, value);
+            }
+        }
+        Err(err) => {
+            info!(
+                "Cache file is failed to open {:?}: {}",
+                cache_file_path, err
+            );
         }
     }
     Ok(())
 }
 
 fn save_cache() -> IOResult<()> {
-    let cache_file_path = cache_dir_path_of("query-cache.json")?;
-
+    let cache_file_path = cache_dir_path_of(CACHE_FILE_NAME)?;
+    let cache_tempfile_path = cache_dir_path_of(CACHE_TEMPFILE_NAME)?;
     let cache_file_lock_result = CACHE_FILE_LOCK.try_lock();
     if cache_file_lock_result.is_err() {
+        info!(
+            "Cache file is locked, cannot save to {:?} now",
+            cache_file_path
+        );
         return Ok(());
     }
+    if let Err(err) = _save_cache(&cache_tempfile_path) {
+        warn!("Failed to save cache {:?}: {}", cache_file_path, err);
+    } else {
+        info!("Save cache to {:?} successfully", cache_file_path);
+        if let Err(err) = rename_file(&cache_tempfile_path, &cache_file_path) {
+            warn!(
+                "Failed to move cache file from {:?} to {:?}: {}",
+                cache_tempfile_path, cache_file_path, err
+            );
+        } else {
+            info!(
+                "Move cache from {:?} to {:?} successfully",
+                cache_tempfile_path, cache_file_path
+            );
+        }
+    }
+    return Ok(());
 
-    let mut cache_file = OpenOptions::new()
-        .write(true)
-        .create(true)
-        .truncate(true)
-        .open(&cache_file_path)?;
-    json_to_writer(&mut cache_file, &*CACHE_MAP)
-        .map_err(|err| IOError::new(IOErrorKind::Other, err))?;
-    Ok(())
+    #[inline]
+    fn _save_cache(cache_file_path: &Path) -> anyhow::Result<()> {
+        let mut cache_file = OpenOptions::new()
+            .write(true)
+            .create(true)
+            .truncate(true)
+            .open(cache_file_path)?;
+        json_to_writer(&mut cache_file, &*CACHE_MAP)
+            .map_err(|err| IOError::new(IOErrorKind::Other, err))?;
+        Ok(())
+    }
 }
 
 #[cfg(test)]
@@ -358,7 +417,7 @@ mod tests {
     use super::{
         super::{
             base::credential::Credential,
-            dot::{DotRecordKey, DotRecords, DotRecordsDashMap},
+            dot::{DotRecordKey, DotRecords, DotRecordsDashMap, DOT_FILE_NAME},
         },
         *,
     };
@@ -443,6 +502,11 @@ mod tests {
                               "domains": [
                                 "iovip.qbox.me"
                               ]
+                            },
+                            "uc": {
+                              "domains": [
+                                "uc.qbox.me"
+                              ]
                             }
                         }]
                     })
@@ -486,12 +550,11 @@ mod tests {
                 let host_selector =
                     HostSelector::builder(vec!["http://".to_owned() + &uc_addr.to_string()])
                         .build();
-                let io_urls = HostsQuerier::new(host_selector, 1, dotter).query_for_io_urls(
-                    ACCESS_KEY,
-                    BUCKET_NAME,
-                    false,
-                )?;
-                assert_eq!(io_urls, vec!["http://iovip.qbox.me".to_owned()]);
+                let querier = HostsQuerier::new(host_selector, 1, dotter);
+                let io_urls = querier.query_for_io_urls(ACCESS_KEY, BUCKET_NAME, false)?;
+                assert_eq!(&io_urls, &["http://iovip.qbox.me".to_owned()]);
+                assert_eq!(&querier.uc_selector.hosts(), &["uc.qbox.me".to_owned()]);
+                assert_eq!(&querier.uc_selector.select_host().host, "uc.qbox.me");
                 sleep(Duration::from_secs(5));
                 assert_eq!(monitor_called.load(Relaxed), 1);
                 Ok(())
@@ -528,6 +591,9 @@ mod tests {
                                   "domains": [
                                     "iovip.qbox.me"
                                   ]
+                                },
+                                "uc": {
+                                  "domains": []
                                 }
                             }]
                         })
@@ -608,7 +674,7 @@ mod tests {
     }
 
     fn clear_cache() -> IOResult<()> {
-        let cache_file_path = cache_dir_path_of("query-cache.json")?;
+        let cache_file_path = cache_dir_path_of(CACHE_FILE_NAME)?;
         std::fs::remove_file(&cache_file_path).or_else(|err| {
             if err.kind() == IOErrorKind::NotFound {
                 Ok(())
@@ -616,7 +682,7 @@ mod tests {
                 Err(err)
             }
         })?;
-        let dot_file_path = cache_dir_path_of("dot-file")?;
+        let dot_file_path = cache_dir_path_of(DOT_FILE_NAME)?;
         std::fs::remove_file(&dot_file_path).or_else(|err| {
             if err.kind() == IOErrorKind::NotFound {
                 Ok(())
